#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use File::Basename;


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
## Date: 
## Goal: 
my $location = $0;
my $title = "hm32mach: convert HapMap3 phased files to MACH format files";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(yunli\@med.unc.edu)\n\n";

## Input Parameters
## 

## Input Files
##

## Output Files

## Note:
## (1) didn't check if positions are strictly increasing

################# END OF DOCUMENTATION ########################
###############################################################


my $MIN_NUM_OPTS = 2;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
	&usage();
	die "Failed to parse parameters\n\n";
}


my %opts = ();

	# Default Optional Options

Getopt::Long::GetOptions(\%opts,qw(
	i=s
	o=s
)) || die "Failed to parse options\n\n";


&printPar();

my $NUM_HEADING_COLUMNS = 2; # number of heading columns: rsID and position_b36
my @OrigAlleles = ();

&main();


# begin sub

sub usage
{
	print "\n";
	print "Usage:\n\t";
	print "-i\t Input (Required) \n\t";
	print "-o\t Output Prefix (Required) \n\t";
	print "\n";
}


sub printPar
{
	print "\n";
	print "Parameters in Effect:\n";
	print "\t Input \n\t\t-i '$opts{i}'\n";
	print "\t Output Prefix \n\t\t-o '$opts{o}'\n";
	print "\n";
}





sub rm
{
	if(-e $_[0])
	{
		print "WARNING: $_[0] existed and removed!!\n\n";
		system "/bin/rm -rf $_[0]";
	}
}



sub bakrm
{
	if(-e $_[0])
	{
		my $tt = $_[0].".old";
		print "WARNING: $_[0] existed and moved to $tt !!\n\n";
		system "mv $_[0] $tt";
	}
}





sub main
{
  my $osnpf = $opts{o}.".snps";
	&rm($osnpf);
  open(OUT, ">>$osnpf") || die "Can't create file $osnpf\n\n";
  if ($opts{i} =~ /(.+)\.gz$/)
  {        
    $opts{i} = $1;
  } # Remove gz suffix if provided
  if (-r "$opts{i}.gz")
  {        
    $opts{i} = "$opts{i}.gz";
    open(IN, "gunzip -c $opts{i} |") || die "Unable to open compressed file '$opts{i}': $!\n";
  }
  else
  {           
    open(IN, $opts{i}) || die "Unable to open file '$opts{i}': $!\n";
  }

	my $firstline = <IN>;
	chomp($firstline);
	my @firstlinearr = (split(/\s+/, $firstline));
	my $numhaplos = scalar(@firstlinearr) - $NUM_HEADING_COLUMNS;
	
	my $linenum = 1;
	while(my $line = <IN>)
	{
  	$linenum ++;
		chomp($line);
		my @linearr = (split(/\s+/,$line));
		if (scalar(@linearr) != scalar(@firstlinearr))
		{
  		print "ERROR: line $linenum in $opts{i} have unexpected number of fields\n\n";
  		exit;
		}
		
		print OUT "$linearr[0]\n";
		
		for (my $i = $NUM_HEADING_COLUMNS; $i < scalar(@linearr); $i++)
		{
  		push @OrigAlleles, $linearr[$i];
		}
	}
	close(IN);
	close(OUT);
	my $numSNPs = $linenum - 1;
	print "Read $numhaplos haplotypes with $numSNPs SNPs\n\n";
	
  my $ohapf = $opts{o}.".hap";
  &rm($ohapf);
  open(OUT, ">>$ohapf") || die "Can't create file $ohapf\n\n";
  for(my $i = 0; $i < $numhaplos; $i ++)
  {
    print OUT "$firstlinearr[$NUM_HEADING_COLUMNS + $i] $firstlinearr[$NUM_HEADING_COLUMNS + $i] ";
    for (my $j = 0; $j < $numSNPs; $j ++)
    {
      print OUT "$OrigAlleles[$j*$numhaplos + $i]" ;
    }
    print OUT "\n";
  }
  close(OUT);
	
}









